#!perl

use Test::More tests => 1;

BEGIN {
    use_ok( 'Business::CompanyDesignator' ) || print "Bail out!
";
}

#diag( "Testing Business::CompanyDesignator $Business::CompanyDesignator::VERSION, Perl $], $^X" );
